@@ -1,11 +1,10 @@
 name = 'lk_utils'
 
-__version__ = '1.4.2'
+__version__ = '1.4.3'
 
 __all__ = [
     "char_converter",
     "chinese_name_processor",
-    "data_convert",
     "easy_launcher",
     "excel_reader",
     "excel_writer",
@@ -14,10 +13,8 @@ __all__ = [
     "lk_browser",
     "lk_config",
     "lk_logger",
-    "lk_wrapper",
     "name_formatter",
     "read_and_write",
-    "regex_helper",
     "time_utils",
     "toolbox",
     "tree_and_trie",
