@@ -1,9 +1,9 @@
 """
 @Author   : Likianta <likianta@foxmail.com>
 @FileName : _typing.py
-@Version  : 0.1.3
+@Version  : 0.1.4
 @Created  : 2020-11-17
-@Updated  : 2020-11-22
+@Updated  : 2020-12-13
 @Desc     : 
 """
 from typing import *
@@ -59,11 +59,14 @@ class FilesniffHint:
 
 
 class ReadAndWriteHint:
+    from io import TextIOWrapper
+    FileHandle = TextIOWrapper
+    
     PlainFileTypes = ('.txt', '.html', '.md', '.rst', '.htm', '.ini')
     StructFileTypes = ('.json', '.json5', '.yaml')
     BinaryFileTypes = ('.xlsx', '.xls', '.pdf')
     
+    PlainData = str
+    StructData = Union[dict, list]
     LoadedData = Union[str, list, dict]
-    SerializableData = Union[str, list, tuple, dict, None]
-    SerialikableData = Union[set, Generator]
-    DumpableData = Union[SerializableData, SerialikableData]
+    DumpableData = Union[dict, list, tuple, str, set, None]
