@@ -3,7 +3,7 @@
 @Module  : excel_writer.py
 @Created : 2018-00-00
 @Updated : 2020-11-22
-@Version : 2.3.12
+@Version : 2.3.13
 @Desc    : ExcelWriter is a post-packing implementation based on XlsxWriter.
 """
 from warnings import filterwarnings
@@ -165,14 +165,14 @@ class ExcelWriter:
     def writelnx(self, *row: Hint.CellValue, fmt=None):
         self.writeln(*row, auto_index=True, fmt=fmt)
     
-    def writerow(self, rowx: int, data: Hint.CellValue, offset=0,
+    def writerow(self, rowx: int, data: Hint.RowValues, offset=0,
                  purify_values=False, fmt=None):
         """ Write row of data to cells. """
         if purify_values:
             data = self.purify_values(data)
         self.sheet.write_row(rowx, offset, data, fmt)
     
-    def writecol(self, colx: int, data: Hint.CellValue, offset=0,
+    def writecol(self, colx: int, data: Hint.ColValues, offset=0,
                  purify_values=False, fmt=None):
         """ Write column of data to cells. """
         if purify_values:
@@ -289,7 +289,7 @@ class ExcelWriter:
 
     # --------------------------------------------------------------------------
 
-    def save(self, alt=False):
+    def save(self, alt=False, open_after_saved=False):
         from xlsxwriter.exceptions import FileCreateError
         try:
             self.book.close()
@@ -309,11 +309,16 @@ class ExcelWriter:
                 self.book.close()
             else:
                 raise e
-        from .lk_logger import lk
-        lk.logt(
-            '[ExcelWriter][D1139]',
-            f'\n\tExcel saved to "{self.filepath}:0"',
-            h=self.__h
-        )
+        
+        if open_after_saved:
+            import os
+            os.startfile(os.path.abspath(self.filepath))
+        else:
+            from .lk_logger import lk
+            lk.logt(
+                '[ExcelWriter][D1139]',
+                f'\n\tExcel saved to "{self.filepath}:0"',
+                h=self.__h
+            )
 
     close = save
